/************************************************
 *     vector.h                            
 ************************************************/

#ifndef _Vector_
typedef struct {
    double x, y, z;
} Vector;
#define _Vector_
#endif

#define VSet3f(v, X, Y, Z) \
       {(v)->x=(X); (v)->y=(Y); (v)->z=(Z);}

#define VAdd(v, a, b) \
       {(v)->x=(a)->x+(b)->x; (v)->y=(a)->y+(b)->y; (v)->z=(a)->z+(b)->z;}

#define VAdd3f(v, a, X, Y, Z) \
       {(v)->x=(a)->x + (X); (v)->y=(a)->y + (Y); (v)->z=(a)->z + (Z);}

#define VSub(v, a, b) \
       {(v)->x=(a)->x-(b)->x; (v)->y=(a)->y-(b)->y; (v)->z=(a)->z-(b)->z;}

#define VSub3f(v, a, X, Y, Z) \
       {(v)->x=(a)->x - (X); (v)->y=(a)->y - (Y); (v)->z=(a)->z - (Z);}

#define VScale(v, a, k) \
       {(v)->x=(a)->x*(k); (v)->y=(a)->y*(k); (v)->z=(a)->z*(k);}

#define VScale3f(v, a, k, l, m) \
       {(v)->x=(a)->x*(k); (v)->y=(a)->y*(l); (v)->z=(a)->z*(m);}

#define VLinComb2(v, a, b, k, l) \
       {(v)->x=(a)->x*(k)+(b)->x*(l); \
        (v)->y=(a)->y*(k)+(b)->y*(l); \
	(v)->z=(a)->z*(k)+(b)->z*(l);}

#define VScaleWithCenter(v, a, c, k) \
       {(v)->x=(a)->x*(k)+(c)->x*(1-(k)); \
        (v)->y=(a)->y*(k)+(c)->y*(1-(k)); \
	(v)->z=(a)->z*(k)+(c)->z*(1-(k));}

#define VLinComb3(v, a, b, c, k, l, m) \
       {(v)->x=(a)->x*(k)+(b)->x*(l)+(c)->x*(m); \
        (v)->y=(a)->y*(k)+(b)->y*(l)+(c)->y*(m); \
	(v)->z=(a)->z*(k)+(b)->z*(l)+(c)->z*(m);}

#define VLinComb4(v, a, b, c, d, k, l, m, n) \
       {(v)->x=(a)->x*(k)+(b)->x*(l)+(c)->x*(m)+(d)->x*(n); \
        (v)->y=(a)->y*(k)+(b)->y*(l)+(c)->y*(m)+(d)->y*(n); \
        (v)->z=(a)->z*(k)+(b)->z*(l)+(c)->z*(m)+(d)->z*(n);}

#define VInner(a, b) \
       ((a)->x*(b)->x+(a)->y*(b)->y+(a)->z*(b)->z)

#define VInner3f(a, k, l, m) \
       ((a)->x*(k)+(a)->y*(l)+(a)->z*(m))

#define VCross(v, a, b) \
       {(v)->x=(a)->y*(b)->z-(a)->z*(b)->y; \
        (v)->y=(a)->z*(b)->x-(a)->x*(b)->z; \
        (v)->z=(a)->x*(b)->y-(a)->y*(b)->x;}

#define VSquare(a) \
       ((a)->x*(a)->x+(a)->y*(a)->y+(a)->z*(a)->z)

#define VLength(a) \
       (sqrt((a)->x*(a)->x+(a)->y*(a)->y+(a)->z*(a)->z))

#define VDist(a, b) \
       (sqrt(((a)->x-(b)->x)*((a)->x-(b)->x)+\
             ((a)->y-(b)->y)*((a)->y-(b)->y)+\
             ((a)->z-(b)->z)*((a)->z-(b)->z)))
      
#define VNormalize(v) \
       {double l; if ((l=VLength(v)) > 0.0)  VScale(v,v,1.0/l);}

#define VCrossXY(a, b) \
       (((a)->x)*((b)->y)-((a)->y)*((b)->x))

#define AreaXY(p1, p2, p3) \
       (((p2)->x-(p1)->x)*((p3)->y-(p1)->y) - \
        ((p2)->y-(p1)->y)*((p3)->x-(p1)->x))

/** Matrix **/
typedef double Matrix[3][3];

#define SetMatrix(M, A) \
      {(M)[0][0]=(A)[0][0];(M)[0][1]=(A)[0][1];(M)[0][2]=(A)[0][2];\
       (M)[1][0]=(A)[1][0];(M)[1][1]=(A)[1][1];(M)[1][2]=(A)[1][2];\
       (M)[2][0]=(A)[2][0];(M)[2][1]=(A)[2][1];(M)[2][2]=(A)[2][2];}

#define MatVec(v, M, a) \
      {Vector _v=*a; \
       (v)->x = (M)[0][0]*_v.x + (M)[0][1]*_v.y + (M)[0][2]*_v.z;\
       (v)->y = (M)[1][0]*_v.x + (M)[1][1]*_v.y + (M)[1][2]*_v.z;\
       (v)->z = (M)[2][0]*_v.x + (M)[2][1]*_v.y + (M)[2][2]*_v.z;}

#define GetRotMatrix(R, axis, theta) \
      {double l=VLength(axis),u=(axis)->x/l,v=(axis)->y/l,w=(axis)->z/l,\
       ct= cos(theta),st= sin(theta); \
       R[0][0]=u*u+(1-u*u)*ct;R[0][1]=u*v*(1-ct)-w*st;R[0][2]=v*w*(1-ct)+v*st;\
       R[1][0]=u*v*(1-ct)+w*st;R[1][1]=v*v+(1-v*v)*ct;R[1][2]=v*w*(1-ct)-u*st;\
       R[2][0]=v*w*(1-ct)-v*st;R[2][1]=v*w*(1-ct)+u*st;R[2][2]=w*w+(1-w*w)*ct;}

#define VRotateAxis(a, axis, theta) \
      {Matrix R; \
       double l=VLength(axis),u=(axis)->x/l,v=(axis)->y/l,w=(axis)->z/l,\
       ct=cos(theta),st=sin(theta); \
       R[0][0]=u*u+(1-u*u)*ct;R[0][1]=u*v*(1-ct)-w*st;R[0][2]=v*w*(1-ct)+v*st;\
       R[1][0]=u*v*(1-ct)+w*st;R[1][1]=v*v+(1-v*v)*ct;R[1][2]=v*w*(1-ct)-u*st;\
       R[2][0]=v*w*(1-ct)-v*st;R[2][1]=v*w*(1-ct)+u*st;R[2][2]=w*w+(1-w*w)*ct;\
       MatVec(a, R, a);}
